"""
Classifies: CHEBI:18390 N-acyl-beta-D-galactosylsphingosine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
from rdkit.Chem import rdMolDescriptors

def is_N_acyl_beta_D_galactosylsphingosine(smiles: str):
    """
    Determines if a molecule is an N-acyl-beta-D-galactosylsphingosine.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an N-acyl-beta-D-galactosylsphingosine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of galactose moiety
    galactose_pattern = Chem.MolFromSmarts('[CH2]O[CH]1O[CH](CO)[CH](O)[CH](O)[CH]1O')
    if not mol.HasSubstructMatch(galactose_pattern):
        return False, "Missing beta-D-galactosyl group"

    # Check for sphingosine backbone with N-acyl group
    sphingosine_pattern = Chem.MolFromSmarts('[CH2]O[CH]([CH](O)/C=C/CCCCC)[NH]C(=O)CCCCC')
    if not mol.HasSubstructMatch(sphingosine_pattern):
        return False, "Missing sphingosine backbone with N-acyl group"

    # Check stereochemistry of key carbons
    # Get matches for the galactose and sphingosine patterns
    gal_matches = mol.GetSubstructMatches(galactose_pattern)
    sph_matches = mol.GetSubstructMatches(sphingosine_pattern)
    
    if not gal_matches or not sph_matches:
        return False, "Could not verify stereochemistry"

    # Check for beta configuration at anomeric carbon
    anomeric_carbon_idx = gal_matches[0][2]  # Index of anomeric carbon in match
    anomeric_carbon = mol.GetAtomWithIdx(anomeric_carbon_idx)
    
    # Check for correct stereochemistry of galactose carbons
    for atom_idx in gal_matches[0]:
        atom = mol.GetAtomWithIdx(atom_idx)
        if atom.GetChiralTag() == Chem.ChiralType.CHI_UNSPECIFIED:
            continue
        if atom.GetChiralTag() != Chem.ChiralType.CHI_TETRAHEDRAL_CW and \
           atom.GetChiralTag() != Chem.ChiralType.CHI_TETRAHEDRAL_CCW:
            return False, "Incorrect stereochemistry in galactose moiety"

    # Verify presence of long chain fatty acid in N-acyl group
    acyl_pattern = Chem.MolFromSmarts('C(=O)CCCCCCCCC')
    if not mol.HasSubstructMatch(acyl_pattern):
        return False, "N-acyl group does not contain required long chain fatty acid"

    return True, "Contains beta-D-galactosyl group, sphingosine backbone, and N-acyl group with correct stereochemistry"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18390',
                          'name': 'N-acyl-beta-D-galactosylsphingosine',
                          'definition': 'An N-acyl-D-galactosylsphingosine in '
                                        'which the anomeric configuration of '
                                        'the galactosyl residue is beta; '
                                        'sphingosine substituted at the O-1 '
                                        'position by a beta-D-galactosyl group '
                                        'and at the N-2 position by an acyl '
                                        'group.',
                          'parents': ['CHEBI:143593', 'CHEBI:83866']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183906,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999836875846206}


"""
Classifies: CHEBI:25380 acetylenic fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import FindMolChiralCenters

def is_acetylenic_fatty_acid(smiles: str):
    """
    Determines if a molecule is an acetylenic fatty acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an acetylenic fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Check for carboxylic acid group
    carboxylic_acid_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_acid_pattern):
        return False, "No carboxylic acid group found"
        
    # Check for triple bond
    triple_bond_pattern = Chem.MolFromSmarts('C#C')
    if not mol.HasSubstructMatch(triple_bond_pattern):
        return False, "No triple bond found"
        
    # Check for carbon chain length (minimum 8 carbons for fatty acids)
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 8:
        return False, "Carbon chain too short for fatty acid classification"
        
    # Count triple bonds
    triple_bonds = len(mol.GetSubstructMatches(triple_bond_pattern))
    
    # Count double bonds
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    double_bonds = len(mol.GetSubstructMatches(double_bond_pattern))
    
    # Check if molecule is linear (allowing for small rings)
    ring_info = mol.GetRingInfo()
    has_large_rings = any(len(ring) > 4 for ring in ring_info.AtomRings())
    
    # Get chiral centers if any
    chiral_centers = FindMolChiralCenters(mol)
    chirality_info = f" with {len(chiral_centers)} chiral center(s)" if chiral_centers else ""
    
    # Build classification reason
    unsaturation_info = []
    if triple_bonds > 0:
        unsaturation_info.append(f"{triple_bonds} triple bond(s)")
    if double_bonds > 0:
        unsaturation_info.append(f"{double_bonds} double bond(s)")
        
    structure_type = "cyclic" if has_large_rings else "linear"
    
    reason = f"{structure_type.capitalize()} fatty acid with {' and '.join(unsaturation_info)}" + \
             f"{chirality_info}, containing {carbon_count} carbons"
    
    return True, reason


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25380',
                          'name': 'acetylenic fatty acid',
                          'definition': 'Any  unsaturated fatty acid '
                                        'containing at least one triple bond '
                                        'in the carbon chain framework.',
                          'parents': ['CHEBI:27208', 'CHEBI:73474']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 63254,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9984216424388781}


"""
Classifies: CHEBI:25848 pantothenic acids
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_pantothenic_acids(smiles: str):
    """
    Determines if a molecule is a pantothenic acid, which is defined as a class of amides formed from pantoic acid and beta-alanine and its derivatives.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a pantothenic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find pantoic acid and beta-alanine substructures
    pantoic_acid_smarts = "[#6](-[#8])(-[#6])(-[#6])(-[#6](-[#6])=[#8])" # Pantoic acid substructure
    beta_alanine_smarts = "[#6](-[#6])(-[#6])(-[#8]-[#6](-[#6])=[#8])" # Beta-alanine substructure

    pantoic_acid_match = mol.GetSubstructMatches(Chem.MolFromSmarts(pantoic_acid_smarts))
    beta_alanine_match = mol.GetSubstructMatches(Chem.MolFromSmarts(beta_alanine_smarts))

    if not pantoic_acid_match or not beta_alanine_match:
        return False, "Missing pantoic acid or beta-alanine substructure"

    # Check for amide bond between pantoic acid and beta-alanine
    for p_idx in pantoic_acid_match:
        for b_idx in beta_alanine_match:
            p_atom = mol.GetAtomWithIdx(p_idx)
            b_atom = mol.GetAtomWithIdx(b_idx)
            if p_atom.GetBonds()[0].GetBondType() == Chem.BondType.AMIDE:
                if b_atom.GetBonds()[0].GetBondType() == Chem.BondType.AMIDE:
                    if p_atom.GetBonds()[0].GetOtherAtomIdx(p_idx) == b_idx:
                        return True, "Molecule is a pantothenic acid"

    return False, "No amide bond found between pantoic acid and beta-alanine substructures"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25848',
                          'name': 'pantothenic acids',
                          'definition': 'A class of amides formed from pantoic '
                                        'acid and beta-alanine and its '
                                        'derivatives.',
                          'parents': ['CHEBI:22823', 'CHEBI:37622']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.GetAtomWithIdx(Mol, tuple)\n'
             'did not match C++ signature:\n'
             '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int idx)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:21731 N-glycosyl compound
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
import re

def is_N_glycosyl_compound(smiles: str):
    """
    Determines if a molecule is an N-glycosyl compound.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an N-glycosyl compound, False otherwise
        str: Reason for classification
    """
    
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return None, None
            
        # Look for sugar ring patterns
        sugar_pattern = Chem.MolFromSmarts('[CR1]1[CR1][CR1][CR1][CR1]O1') # pyranose
        furanose_pattern = Chem.MolFromSmarts('[CR1]1[CR1][CR1][CR1]O1') # furanose
        
        sugar_matches = mol.GetSubstructMatches(sugar_pattern)
        furanose_matches = mol.GetSubstructMatches(furanose_pattern)
        
        if not (sugar_matches or furanose_matches):
            return False, "No sugar ring found"
            
        # Get all sugar ring atoms
        sugar_atoms = set()
        for match in sugar_matches:
            sugar_atoms.update(match)
        for match in furanose_matches:
            sugar_atoms.update(match)
            
        # Look for N-glycosidic bonds
        for atom_idx in sugar_atoms:
            atom = mol.GetAtomWithIdx(atom_idx)
            if atom.GetSymbol() == 'O': # anomeric oxygen
                for neighbor in atom.GetNeighbors():
                    if neighbor.GetIdx() not in sugar_atoms:
                        # Check if this non-sugar atom is connected to N
                        for n2 in neighbor.GetNeighbors():
                            if n2.GetSymbol() == 'N':
                                return True, "N-glycosidic bond found"
                                
        return False, "No N-glycosidic bond found"
        
    except Exception as e:
        return None, f"Error: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:21731',
                          'name': 'N-glycosyl compound',
                          'definition': 'A glycosyl compound arising formally '
                                        'from the elimination of water from a '
                                        'glycosidic hydroxy group and an H '
                                        'atom bound to a nitrogen atom, thus '
                                        'creating a C-N bond.',
                          'parents': [   'CHEBI:35352',
                                         'CHEBI:63161',
                                         'CHEBI:63299']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183221,
    'num_false_negatives': 71,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9996126399406412}


"""
Classifies: CHEBI:24281 glucosyl group
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_glucosyl_group(smiles: str):
    """
    Determines if a molecule is a glucosyl group, which is a glycosyl group obtained by removing
    the hydroxy group from the hemiacetal function of a glucose and, by extension, of a lower
    oligosaccharide having glucose at the reducing end.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a glucosyl group, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains a glucose substructure
    glucose_smarts = '[C@H]1([C@@H]([C@H]([C@@H]([C@@H]([C@@H](O1)O)O)O)O)O'
    glucose_match = mol.HasSubstructMatch(Chem.MolFromSmarts(glucose_smarts))

    if not glucose_match:
        return False, "The molecule does not contain a glucose substructure"

    # Check if the glucose substructure is at the reducing end
    reducing_end = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetDegree() == 1 and atom.IsInRing():
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'C' and neighbor.GetDegree() == 4:
                    reducing_end = True
                    break

    if not reducing_end:
        return False, "The glucose substructure is not at the reducing end"

    # Check if the hemiacetal hydroxyl group is removed
    hemiacetal_hydroxyl_removed = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetDegree() == 2 and atom.IsInRing():
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'C' and neighbor.GetDegree() == 4:
                    hemiacetal_hydroxyl_removed = True
                    break

    if not hemiacetal_hydroxyl_removed:
        return False, "The hemiacetal hydroxyl group is not removed"

    return True, "The molecule is a glucosyl group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24281',
                          'name': 'glucosyl group',
                          'definition': 'A glycosyl group obtained by removing '
                                        'the hydroxy group from the hemiacetal '
                                        'function of a glucose and, by '
                                        'extension, of a lower oligosaccharide '
                                        'having glucose at the reducing end.',
                          'parents': ['CHEBI:24403']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.HasSubstructMatch(Mol, NoneType)\n'
             'did not match C++ signature:\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'RDKit::SubstructMatchParameters params=True)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'RDKit::SubstructMatchParameters params)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:140356 7-methoxyisoflavones
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_7_methoxyisoflavone(smiles: str):
    """
    Determines if a molecule is a 7-methoxyisoflavone.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a 7-methoxyisoflavone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the isoflavone moiety
    isoflavone_pattern = Chem.MolFromSmarts('C1=C(C(=O)C2=C(C=C1)C(=O)C3=C(C=C(C=C3)O)O2)O')
    matches = mol.GetSubstructMatches(isoflavone_pattern)

    if not matches:
        return False, "Molecule does not contain an isoflavone moiety"

    # Check for a methoxy group at the 7-position
    for match in matches:
        ring_atoms = list(match)
        ring_atoms.sort()  # Ensure consistent ordering
        ring_info = mol.GetRingInfo().AtomRings()[ring_atoms.index(tuple(ring_atoms))]

        # Find the 7-position atom
        for i, atom_idx in enumerate(ring_info):
            atom = mol.GetAtomWithIdx(atom_idx)
            if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 0:
                break
        else:
            continue

        neighbor = atom.GetNeighbors()[0]
        if neighbor.GetSymbol() == 'C' and neighbor.GetTotalNumHs() == 0:
            methoxy_neighbor = neighbor.GetNeighbors()[0]
            if methoxy_neighbor.GetSymbol() == 'O' and methoxy_neighbor.GetTotalNumHs() == 1:
                return True, "Molecule is a 7-methoxyisoflavone"

    return False, "Molecule does not have a methoxy group at the 7-position of the isoflavone moiety"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140356',
                          'name': '7-methoxyisoflavones',
                          'definition': 'Any methoxyisoflavone that has a '
                                        'methoxy group at the 7-position of '
                                        'the isoflavone moiety.',
                          'parents': ['CHEBI:38756']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "name 'is_7_methoxyisoflavones' is not defined",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:26666 short-chain fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_short_chain_fatty_acid(smiles: str):
    """
    Determines if a molecule is a short-chain fatty acid (aliphatic monocarboxylic acid with <6 carbons).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a short-chain fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for exactly one carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)O')
    matches = mol.GetSubstructMatches(carboxylic_pattern)
    if len(matches) == 0:
        return False, "No carboxylic acid group found"
    if len(matches) > 1:
        return False, "Multiple carboxylic acid groups found"

    # Count total carbons
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count > 5:
        return False, f"Carbon chain too long ({carbon_count} carbons)"

    # Check if aliphatic (no aromatic rings)
    if any(atom.GetIsAromatic() for atom in mol.GetAtoms()):
        return False, "Contains aromatic rings"

    # Check for non-hydrocarbon substituents (besides the carboxylic acid group)
    allowed_atoms = {'C', 'H', 'O'}
    for atom in mol.GetAtoms():
        if atom.GetSymbol() not in allowed_atoms:
            return False, f"Contains non-hydrocarbon substituent ({atom.GetSymbol()})"

    # Count oxygen atoms - should only be 2 from carboxylic acid group
    oxygen_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if oxygen_count > 2:
        return False, "Contains additional oxygen substituents"

    # All checks passed
    return True, f"Valid short-chain fatty acid with {carbon_count} carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26666',
                          'name': 'short-chain fatty acid',
                          'definition': 'An aliphatic monocarboxylic acid with '
                                        'a chain length of less than C6. If '
                                        'any non-hydrocarbon substituent is '
                                        'present, the compound is not normally '
                                        'regarded as a short-chain fatty acid.',
                          'parents': ['CHEBI:35366']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 78,
    'num_true_negatives': 183804,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.012658227848101266,
    'recall': 0.2,
    'f1': 0.023809523809523808,
    'accuracy': 0.9995540739693398}


"""
Classifies: CHEBI:134400 cationic peptidyl group
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_cationic_peptidyl_group(smiles: str):
    """
    Determines if a molecule contains a cationic peptidyl group.
    A cationic peptidyl group is defined as an organic cationic group obtained by protonation 
    of any peptide group; major species at pH 7.3.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains a cationic peptidyl group, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of peptide bonds (C(=O)N)
    peptide_pattern = Chem.MolFromSmarts('[C](=O)[NH]')
    if not mol.HasSubstructMatch(peptide_pattern):
        return False, "No peptide bonds found"

    # Check for presence of cationic groups
    cationic_patterns = [
        Chem.MolFromSmarts('[NH3+]'),  # Protonated amine
        Chem.MolFromSmarts('[NH2+]'),  # Protonated imine
        Chem.MolFromSmarts('[N+](C)(C)(C)'), # Quaternary amine
    ]
    
    cationic_groups = []
    for pattern in cationic_patterns:
        if mol.HasSubstructMatch(pattern):
            matches = mol.GetSubstructMatches(pattern)
            cationic_groups.extend(matches)
    
    if not cationic_groups:
        return False, "No cationic groups found"

    # Check for amino acid backbone pattern
    aa_pattern = Chem.MolFromSmarts('[NH2,NH3+,N+0,N+1][CH]C(=O)')
    if not mol.HasSubstructMatch(aa_pattern):
        return False, "No amino acid backbone found"

    # Count formal charges
    total_charge = sum(atom.GetFormalCharge() for atom in mol.GetAtoms())
    if total_charge <= 0:
        return False, "Total molecular charge is not positive"

    # Check for presence of wildcards (*) indicating this is a residue/group
    if '*' not in smiles:
        return False, "Not a residue/group (no wildcards found)"

    return True, f"Cationic peptidyl group with {len(cationic_groups)} cationic centers and total charge of +{total_charge}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134400',
                          'name': 'cationic peptidyl group',
                          'definition': 'An organic cationic group obtained by '
                                        'protonation of any peptide group; '
                                        'major species at pH 7.3.',
                          'parents': ['CHEBI:64769']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 25,
    'num_true_negatives': 183878,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.10714285714285714,
    'recall': 1.0,
    'f1': 0.19354838709677416,
    'accuracy': 0.9998640609876784}


"""
Classifies: CHEBI:32877 primary amine
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_primary_amine(smiles: str):
    """
    Determines if a molecule is a primary amine.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a primary amine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Get the number of nitrogen atoms
    num_nitrogen = sum(atom.GetAtomicNum() == 7 for atom in mol.GetAtoms())

    # Primary amines have only one nitrogen atom
    if num_nitrogen != 1:
        return False, "Molecule does not have exactly one nitrogen atom"

    # Get the nitrogen atom
    nitrogen_atom = [atom for atom in mol.GetAtoms() if atom.GetAtomicNum() == 7][0]

    # Primary amines have three substituents on the nitrogen atom
    if len(nitrogen_atom.GetNeighbors()) != 3:
        return False, "Nitrogen atom does not have three substituents"

    # One of the substituents should be hydrogen
    if not any(neighbor.GetSymbol() == 'H' for neighbor in nitrogen_atom.GetNeighbors()):
        return False, "Nitrogen atom does not have a hydrogen substituent"

    # Check if the other substituents are hydrocarbyl groups
    hydrocarbyl_groups = []
    for neighbor in nitrogen_atom.GetNeighbors():
        if neighbor.GetSymbol() != 'H':
            hydrocarbyl_group = Chem.MolFragmentToSmiles(mol, [neighbor.GetIdx()], bondsToUse=[mol.GetBondBetweenAtoms(neighbor.GetIdx(), nitrogen_atom.GetIdx()).GetIdx()])
            hydrocarbyl_groups.append(hydrocarbyl_group)

    hydrocarbyl_groups_str = ', '.join(hydrocarbyl_groups)
    return True, f"Primary amine with hydrocarbyl groups: {hydrocarbyl_groups_str}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:32877',
                          'name': 'primary amine',
                          'definition': 'A compound formally derived from '
                                        'ammonia by replacing one hydrogen '
                                        'atom by a hydrocarbyl group.',
                          'parents': ['CHEBI:32952', 'CHEBI:50994']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 5,
    'num_true_negatives': 183769,
    'num_false_negatives': 16,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9998857391588226}


"""
Classifies: CHEBI:15897 N-(long-chain-acyl)ethanolamine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_N__long_chain_acyl_ethanolamine(smiles: str):
    """
    Determines if a molecule is an N-(long-chain-acyl)ethanolamine.
    These are N-acylethanolamines with acyl chain length ≥ C12.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an N-(long-chain-acyl)ethanolamine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of required functional groups
    if not all(group in Chem.MolToSmiles(mol) for group in ['C(=O)N', 'CCO']):
        return False, "Missing required N-acylethanolamine structure"

    # Find the amide carbon
    pattern = Chem.MolFromSmarts('C(=O)NCCO')
    if not mol.HasSubstructMatch(pattern):
        return False, "Missing N-acylethanolamine backbone"

    # Get the amide carbon and trace back the chain
    matches = mol.GetSubstructMatches(pattern)
    amide_carbon_idx = matches[0][0]  # First atom in the SMARTS pattern is the amide carbon
    
    # Function to count carbons in the acyl chain
    def count_chain_carbons(mol, start_idx, visited=None):
        if visited is None:
            visited = set()
        
        count = 1  # Count the current carbon
        visited.add(start_idx)
        
        atom = mol.GetAtomWithIdx(start_idx)
        for neighbor in atom.GetNeighbors():
            n_idx = neighbor.GetIdx()
            if n_idx not in visited and neighbor.GetSymbol() == 'C':
                # Don't count carbons in the ethanolamine part
                if not any(n_idx in [m[3], m[4]] for m in matches):
                    count += count_chain_carbons(mol, n_idx, visited)
        
        return count

    chain_length = count_chain_carbons(mol, amide_carbon_idx)

    if chain_length >= 12:
        return True, f"N-acylethanolamine with chain length C{chain_length}"
    else:
        return False, f"N-acylethanolamine with chain length C{chain_length} (less than C12)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:15897',
                          'name': 'N-(long-chain-acyl)ethanolamine',
                          'definition': 'Any N-acylethanolamine in which the '
                                        'acyl group has a chain length of C12 '
                                        'or greater.',
                          'parents': ['CHEBI:167098']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: invalid syntax (<string>, line 47)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 63521,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9984282660631209}


"""
Classifies: CHEBI:26872 terpene ketone
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors, rdFMCS

def is_terpene_ketone(smiles: str):
    """
    Determines if a molecule is a terpene ketone (any terpenoid containing a keto group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a terpene ketone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains a keto group
    has_keto = any(atom.GetSymbol() == 'O' and atom.GetDegree() == 1 and
                   atom.GetTotalNumHs() == 0 for atom in mol.GetAtoms())
    if not has_keto:
        return False, "No keto group found"

    # Check if the molecule is a terpenoid
    terpenoid_patterns = [Chem.MolFromSmiles('C=CC(C)C'),  # isoprene unit
                          Chem.MolFromSmiles('CC(C)=C')]
    is_terpenoid = any(mol.HasSubstructMatch(pattern) for pattern in terpenoid_patterns)

    if not is_terpenoid:
        return False, "Not a terpenoid"

    return True, "Terpene ketone"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26872',
                          'name': 'terpene ketone',
                          'definition': 'Any terpenoid which contains a keto '
                                        'group.',
                          'parents': ['CHEBI:17087', 'CHEBI:26873']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 16,
    'num_false_positives': 100,
    'num_true_negatives': 634,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.13793103448275862,
    'recall': 0.8,
    'f1': 0.2352941176470588,
    'accuracy': 0.8620689655172413}


"""
Classifies: CHEBI:131860 octadecanoid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_octadecanoid_anion(smiles: str):
    """
    Determines if a molecule is an octadecanoid anion.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an octadecanoid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Check for carboxylate anion
    carboxylate = False
    carboxylate_carbon = None
    for atom in mol.GetAtoms():
        if atom.GetFormalCharge() == -1 and atom.GetSymbol() == 'O':
            # Check if connected to C=O
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'C':
                    for bond in neighbor.GetBonds():
                        other_atom = bond.GetOtherAtom(neighbor)
                        if other_atom.GetSymbol() == 'O' and bond.GetBondType() == Chem.BondType.DOUBLE:
                            carboxylate = True
                            carboxylate_carbon = neighbor.GetIdx()
                            break
    
    if not carboxylate:
        return False, "No carboxylate anion group found"

    # Check carbon count (should be 18 for octadecanoid)
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 18:
        return False, f"Carbon count is {carbon_count}, should be 18"
    
    # Check for longest carbon chain starting from carboxylate
    if carboxylate_carbon is not None:
        # Create a path of length 18 from carboxylate carbon
        paths = Chem.FindAllPathsOfLengthN(mol, 18, useBonds=False, useHs=False)
        valid_path = False
        for path in paths:
            if path[0] == carboxylate_carbon:
                # Check if path consists of only carbons
                if all(mol.GetAtomWithIdx(idx).GetSymbol() == 'C' for idx in path):
                    valid_path = True
                    break
        
        if not valid_path:
            return False, "No valid 18-carbon chain found starting from carboxylate group"
        
    # Check for unsaturation (must have at least one double bond)
    double_bonds = 0
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.DOUBLE:
            if bond.GetBeginAtom().GetSymbol() == 'C' and bond.GetEndAtom().GetSymbol() == 'C':
                double_bonds += 1
    
    if double_bonds == 0:
        return False, "No carbon-carbon double bonds found"
    
    return True, f"Octadecanoid anion with {double_bonds} C=C double bond(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131860',
                          'name': 'octadecanoid anion',
                          'definition': 'An unsaturated fatty acid anion '
                                        'obtained by the deprotonation of the '
                                        'carboxy group of any octadecanoid.',
                          'parents': ['CHEBI:2580', 'CHEBI:57560']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.13461538461538464 is too low.\n'
               'True positives: '
               "[('C(CCCCCCC\\\\C=C/C=C/[C@@H](CCCCC)O)(=O)[O-]', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('CCCCCCCCC\\\\C=C/C\\\\C=C/CCCC([O-])=O', 'Octadecanoid anion "
               "with 2 C=C double bond(s)'), "
               "('O1[C@@H](CCCCCCCC([O-])=O)[C@@H]1[C@H](/C=C\\\\CCCCC)O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('[H]C(CCCCCCCC([O-])=O)=CC([H])=CCCCCCC', 'Octadecanoid anion "
               "with 2 C=C double bond(s)'), "
               "('C([O-])(=O)CCCCCCC/C=C\\\\[C@@H]1[C@@H]2[C@@H](C[C@]1([C@H](CC)O2)[H])O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('C(CCC(C/C=C\\\\C/C=C\\\\CC)O)CCCCCC(=O)[O-]', 'Octadecanoid "
               "anion with 2 C=C double bond(s)'), "
               "('CCCCC\\\\C=C/C\\\\C=C/[C@H](O)CC[C@@H](O)CCCC([O-])=O', "
               "'Octadecanoid anion with 2 C=C double bond(s)')]\n"
               "False positives: [('CCCC\\\\C=C/CCCCCCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C\\\\C=C/C=C/C(O)CCCCCCCC([O-])=O', 'Octadecanoid "
               "anion with 3 C=C double bond(s)'), "
               "('C(\\\\CCCCCCC(C(=O)[O-])O)=C\\\\CCCCCCCC', 'Octadecanoid "
               "anion with 1 C=C double bond(s)'), "
               "('[C@@H]1([C@@H](C[C@H]([C@@H](O1)C)O)O)OCCCCCCCCC\\\\C=C\\\\C(=O)[O-]', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CCCCCCC(O)C\\\\C=C\\\\CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCCCCCC(C\\\\C=C\\\\CCCCCCCC([O-])=O)OP([O-])([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('OC(=O)[C@@H]1CC(=C\\\\C=[N+]2/[C@@H](Cc3cc(oc(=O)c23)C(O)=O)C([O-])=O)/C=C(N1)C(O)=O', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('C1([C@H](CCC1=O)CC(N[C@H](C([O-])=O)[C@H](CC)C)=O)C/C=C\\\\CC', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC([O-])=O', "
               "'Octadecanoid anion with 4 C=C double bond(s)'), "
               "('CC\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\CCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 4 C=C double bond(s)'), "
               "('CCCCCC\\\\C=C\\\\CCCCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C=C/C=C/C=C\\\\CCCCCCCC([O-])=O', 'Octadecanoid "
               "anion with 4 C=C double bond(s)'), "
               "('CCCCCCC(C\\\\C=C/CCCCCCCC([O-])=O)OP([O-])([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid "
               "anion with 3 C=C double bond(s)'), "
               "('[C@@]12([C@H](CCC=C1C=C[C@@H]([C@@H]2CC[C@H](C[C@H](CC(=O)[O-])O)O)C)O)[H]', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('CC\\\\C=C/C[C@H]1[C@@H](CCCCCCCC([O-])=O)C=CC1=O', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('CC(C)[C@@H](/C=C(\\\\C)/C(=O)[O-])N(C)C(=O)[C@H](C(C)C)NC(=O)[C@H](CO)NC(N)=[NH2+]', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC\\\\C(C)=C\\\\CSC[C@H]([NH3+])C([O-])=O', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('C(C/C=C\\\\CCCCCCCCCCC)CCC(=O)[O-]', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('S(C1=C(N2C(=O)C(C2C1C)C(O)C)C(=O)O)C3CNC(C3)C(=O)N(C)C.[Na+].[Na+].O=C([O-])[O-]', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('Oc1ccc(CC(OC(=O)\\\\C=C\\\\c2ccc(O)c(O)c2)C([O-])=O)cc1O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('C/1(=C\\\\C2=CC=CC=C2)\\\\O[C@@](CC3=CC=CC=C3)(OC1=O)C(=O)[O-]', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('[C@@H]1([C@H](CCC1=O)CC(N[C@H](C([O-])=O)[C@H](CC)C)=O)C/C=C\\\\CCO', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('C(CCCCCCC/C=C\\\\C(CCCCCCC)O)([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCCCC[C@@H]1O\\\\C1=C/C=C\\\\CCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('[H]C(\\\\C=C/CCCCCCCC([O-])=O)=C1O[C@H]1C\\\\C=C/CC', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('[C@@]12(CCCC=C1C=C[C@@H]([C@@H]2CC[C@H](C[C@H](CC(=O)[O-])O)O)C)[H]', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('CCCCCCCC\\\\C=C\\\\CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid anion with "
               "1 C=C double bond(s)'), "
               "('CC\\\\C=C/C[C@@H]1[C@H](CCCCCCCC([O-])=O)C=CC1=O', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('C([O-])(=O)CCCCCCC[C@H]1C(=C(/C=C\\\\CCCCC)[H])O1', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('[Na+].CCCCCCCC\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCCCC\\\\C=C/CC1OC1CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('[Na+].[Na+].[H][C@]12SCC(CSc3nnnn3CS([O-])(=O)=O)=C(N1C(=O)[C@H]2NC(=O)[C@H](O)c1ccccc1)C([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CCCCCCC\\\\C=C\\\\CCCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCC\\\\C=C/[C@H](OO)\\\\C=C\\\\C=C/CCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('CCCCCC\\\\C=C/CCCCCCCCCC([O-])=O', 'Octadecanoid anion with "
               "1 C=C double bond(s)'), "
               "('C1(=C(C(=CC(=C1)O)O)C([O-])=O)/C=C/CCCC(=O)CCC[C@@H](O)C', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('[H]C(CCCCCC)=C([H])CCCCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('C(=C\\\\C/C=C\\\\CCCCC)\\\\CCCCCC[C@H](C(=O)[O-])OO', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('O=[N+]([O-])C(=CCCCCCCCC)CCCCCCCC(=O)[O-]', 'Octadecanoid "
               "anion with 1 C=C double bond(s)'), "
               "('CCCCC\\\\C=C/CC(O)C(O)CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('[O-]C(CCCCCCC/C=C\\\\CCCCCCC(C)O)=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('C[C@H](CCCCCCC/C=C/C([O-])=O)O[C@@H]1O[C@@H](C)[C@H](O)C[C@H]1O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C[C@@H]1[C@H](CCCCCCCC([O-])=O)CCC1=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('[O-]C(=O)[C@@H]1CC2=CC(=O)C(=O)C=C2N1/C=C/C3=CC(=[NH+][C@@H](C3)C([O-])=O)C([O-])=O', "
               "'Octadecanoid anion with 4 C=C double bond(s)'), "
               "('CCCC\\\\C=C\\\\O\\\\C=C\\\\C=C/CCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('[H][C@]12SCC(CSc3nnnn3C)=C(N1C(=O)[C@H]2NC(=O)[C@H](O)c1ccccc1)C([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C=C/O\\\\C=C\\\\C=C/CCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 4 C=C double bond(s)'), "
               "('[C@@]12(CCCC[C@@]1(C=C[C@@H]([C@@H]2CC[C@H](C[C@H](CC(=O)[O-])O)O)C)[H])[H]', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CCCCCCC(O)C\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('C(CCCCCCC/C=C\\\\CCCCCC)(=O)NCC([O-])=O', 'Octadecanoid "
               "anion with 1 C=C double bond(s)'), "
               "('C(=CCCCCCCCC)CCCCCCCC(=O)[O-]', 'Octadecanoid anion with 1 "
               "C=C double bond(s)'), ('CCCCCC#CC\\\\C=C/CCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C\\\\C=C/C[C@H](OO)\\\\C=C\\\\CCCCCCC([O-])=O', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('CCCCC\\\\C=C/C=C/O\\\\C=C\\\\CCCCCCC([O-])=O', 'Octadecanoid "
               "anion with 3 C=C double bond(s)'), "
               "('C(=O)(CCCC/C=C/CCCCCCCCCCC)[O-]', 'Octadecanoid anion with 1 "
               "C=C double bond(s)'), "
               "('[C@@]12(CCCCC1=C[C@H]([C@@H]([C@@H]2CC[C@H](C[C@H](CC(=O)[O-])O)O)C)O)[H]', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CC(C)=CCc1cccc2Nc3cccc(C([O-])=O)c3Nc12', 'Octadecanoid "
               "anion with 1 C=C double bond(s)'), "
               "('C(=O)([O-])[C@@H](N*)CSC/C=C(/CC/C=C(/CCC=C(C)C)\\\\C)\\\\C', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('CCCCCCCC[C@H](O)\\\\C=C\\\\CCCCCCC([O-])=O', 'Octadecanoid "
               "anion with 1 C=C double bond(s)'), "
               "('CC\\\\C=C/C[C@H]1[C@@H](CCCCCCCC([O-])=O)CCC1=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('[H][C@]12SCC(CSc3nc(=O)c(=O)[nH]n3C)=C(N1C(=O)[C@H]2NC(=O)C(=N/OC)\\\\c1csc(N)n1)C([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('C(=C\\\\C/C=C\\\\CCCC(C)O)\\\\CCCCCCCC(=O)[O-]', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC[C@@H](O)C([O-])=O', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('C(CCCCCCC(/C=C\\\\CCCCCCCC)O)([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCCCC\\\\C=C/CC(O)CCCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('[O-]C(CCCCCCC/C=C\\\\C[C@@H](CCCCCC)O)=O', 'Octadecanoid "
               "anion with 1 C=C double bond(s)'), "
               "('C1([C@H](CCC1=O)CC(N[C@H](C([O-])=O)[C@H](CC)C)=O)C/C=C\\\\CCO', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('C(=C\\\\C/C=C\\\\CCCCCO)\\\\CCCCCCCC(=O)[O-]', 'Octadecanoid "
               "anion with 2 C=C double bond(s)'), "
               "('OCCCCCCCC\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid anion with "
               "1 C=C double bond(s)'), "
               "('C(CCCC([O-])=O)CCC/C=C\\\\C=C\\\\[C@H](C/C=C\\\\CC)O', "
               "'Octadecanoid anion with 3 C=C double bond(s)'), "
               "('CC\\\\C=C/C\\\\C=C/C=C/O\\\\C=C\\\\CCCCCCC([O-])=O', "
               "'Octadecanoid anion with 4 C=C double bond(s)'), "
               "('C(\\\\CCCCCC[C@H](C(=O)[O-])OO)=C\\\\CCCCCCCC', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('[H]C(CCC=C([H])C=C([H])CCCCCCCC([O-])=O)=CCC', 'Octadecanoid "
               "anion with 3 C=C double bond(s)'), "
               "('CCCCC\\\\C=C/C=C/CCCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 2 C=C double bond(s)'), "
               "('[O-]C(CCCCCCC/C=C\\\\C[C@@H](CCCCCCO)O)=O', 'Octadecanoid "
               "anion with 1 C=C double bond(s)'), "
               "('C1(O)=C([C@@](C([O-])=O)(OC1=O)CC2=CC=C(C=C2)O)C3=CC=C(C=C3)O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('Oc1ccc(C[C@@H](OC(=O)\\\\C=C\\\\c2ccc(O)c(O)c2)C([O-])=O)cc1O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CCCCCC(O)C(O)C\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC([O-])=O', 'Octadecanoid "
               "anion with 3 C=C double bond(s)'), "
               "('[H][C@]12SCC(CSc3nnnn3CS([O-])(=O)=O)=C(N1C(=O)[C@H]2NC(=O)[C@H](O)c1ccccc1)C([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('[H]C(CCC)=CCC=C([H])C=C([H])CCCCCCCC([O-])=O', 'Octadecanoid "
               "anion with 3 C=C double bond(s)'), "
               "('[Na+].[H][C@]12SCC(CSc3nnnn3C)=C(N1C(=O)[C@H]2NC(=O)[C@H](O)c1ccccc1)C([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CCCCC[C@@H]1O[C@@H]1C\\\\C=C/CCCCCCCC([O-])=O', "
               "'Octadecanoid anion with 1 C=C double bond(s)'), "
               "('CCCCCC1OC1C\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid anion "
               "with 1 C=C double bond(s)'), "
               "('CCC\\\\C=C/C\\\\C=C\\\\C=C/CCCCCCCC([O-])=O', 'Octadecanoid "
               "anion with 3 C=C double bond(s)'), "
               "('C1[C@]2([C@]3([C@@](/C(/C(CC3)=O)=C/C=C(\\\\O)/C(=O)[O-])(CC[C@@]2(C(=O)C1)C)[H])[H])[H]', "
               "'Octadecanoid anion with 2 C=C double bond(s)'), "
               "('[C@@H]1([C@H](CCC1=O)CC(N[C@H](C([O-])=O)[C@H](CC)C)=O)C/C=C\\\\CC', "
               "'Octadecanoid anion with 1 C=C double bond(s)')]\n"
               "False negatives: [('*C([O-])=O', 'Carbon count is 1, should be "
               "18')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 18,
    'num_true_negatives': 183835,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.18181818181818182,
    'recall': 0.5,
    'f1': 0.26666666666666666,
    'accuracy': 0.9998803443905995}


"""
Classifies: CHEBI:26667 sialic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_sialic_acid(smiles: str):
    """
    Determines if a molecule is a sialic acid (N-acylneuraminic acid or derivative).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a sialic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Core structure requirements:
    # - 9 carbons 
    # - N-acyl group
    # - Carboxylic acid or derivative
    # - Multiple hydroxyl groups
    # - Pyranose ring
    
    # Count carbons
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 9:
        return False, "Does not contain 9 carbons characteristic of sialic acid core"
        
    # Check for N-acyl group
    pattern_nacyl = Chem.MolFromSmarts('[NH][C](=O)')
    if not mol.HasSubstructMatch(pattern_nacyl):
        return False, "Missing N-acyl group"
        
    # Check for carboxylic acid or derivative
    pattern_acid = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    pattern_ester = Chem.MolFromSmarts('[CX3](=O)[OX2][C]')
    if not (mol.HasSubstructMatch(pattern_acid) or mol.HasSubstructMatch(pattern_ester)):
        return False, "Missing carboxylic acid or derivative group"
        
    # Count hydroxyl groups (should have multiple)
    pattern_oh = Chem.MolFromSmarts('[OH]')
    oh_matches = len(mol.GetSubstructMatches(pattern_oh))
    if oh_matches < 3:
        return False, "Insufficient hydroxyl groups"
        
    # Check for pyranose ring
    pattern_pyranose = Chem.MolFromSmarts('O1CCCCC1')
    if not mol.HasSubstructMatch(pattern_pyranose):
        return False, "Missing pyranose ring"

    # If all requirements are met
    return True, "Contains characteristic sialic acid structural features: 9 carbons, N-acyl group, carboxylic acid/derivative, multiple hydroxyls, and pyranose ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26667',
                          'name': 'sialic acid',
                          'definition': 'Any of the N-acylneuraminic acids and '
                                        'their esters and other derivatives of '
                                        'the alcoholic hydroxy groups.',
                          'parents': ['CHEBI:25508']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 2,
    'num_true_negatives': 183911,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999782508223908}


"""
Classifies: CHEBI:134361 allylic alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_allylic_alcohol(smiles: str):
    """
    Determines if a molecule contains an allylic alcohol group.
    An allylic alcohol has a hydroxy group attached to a saturated carbon atom 
    adjacent to a carbon-carbon double bond.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains allylic alcohol, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Pattern for allylic alcohol:
    # [C;!$(C=*);!$(C#*)][OH] adjacent to C=C
    # The central carbon must be sp3 hybridized (not part of double/triple bond)
    # and have a hydroxyl group and be next to a C=C double bond
    pattern = Chem.MolFromSmarts("[C;!$(C=*);!$(C#*);!$(C[O;!H])][OH]-[#6]=[#6]")
    
    if pattern is None:
        return None, "Invalid SMARTS pattern"
        
    # Find all double bonds in the molecule
    double_bonds = mol.GetSubstructMatches(Chem.MolFromSmarts("C=C"))
    if not double_bonds:
        return False, "No carbon-carbon double bonds found"

    # Find all hydroxyl groups attached to sp3 carbons
    hydroxy_groups = mol.GetSubstructMatches(Chem.MolFromSmarts("[C;!$(C=*);!$(C#*)][OH]"))
    if not hydroxy_groups:
        return False, "No hydroxyl groups found on sp3 carbons"

    # Check if any hydroxyl group is adjacent to a double bond
    allylic_alcohols = []
    for oh_match in hydroxy_groups:
        c_with_oh = mol.GetAtomWithIdx(oh_match[0])
        if c_with_oh.GetHybridization() != Chem.HybridizationType.SP3:
            continue
            
        # Check neighbors of the carbon with OH
        for neighbor in c_with_oh.GetNeighbors():
            if neighbor.GetSymbol() != 'C':
                continue
                
            # Check if this carbon is part of a double bond
            for bond in neighbor.GetBonds():
                if bond.GetBondType() == Chem.BondType.DOUBLE:
                    other_atom = bond.GetOtherAtom(neighbor)
                    if other_atom.GetSymbol() == 'C':
                        allylic_alcohols.append(oh_match)
                        break

    if allylic_alcohols:
        count = len(set(allylic_alcohols))  # Remove duplicates
        return True, f"Found {count} allylic alcohol group{'s' if count > 1 else ''}"

    return False, "No allylic alcohol groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134361',
                          'name': 'allylic alcohol',
                          'definition': 'An alcohol where the hydroxy group is '
                                        'attached to a saturated carbon atom '
                                        'adjacent to a double bond (R groups '
                                        'may be H, organyl, etc.).',
                          'parents': ['CHEBI:30879', 'CHEBI:78840']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('C(C(O)=O)C/C=C\\\\C[C@H](\\\\C=C\\\\C=C/C=C/C=C/[C@H]([C@@H](C/C=C\\\\CC)O)O)O', "
               "'No allylic alcohol groups found'), "
               "('C\\\\C(=C/CO)\\\\C=C\\\\C=C(/C)\\\\C=C\\\\C1=C(C)CCCC1(C)C', "
               "'No allylic alcohol groups found'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C\\\\C=C/C=C/[C@H](C/C=C\\\\C/C=C\\\\CC)O', "
               "'No allylic alcohol groups found'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C\\\\C=C/C=C/[C@@H](C/C=C\\\\C/C=C\\\\CC)O', "
               "'No allylic alcohol groups found'), "
               "('C1=2[C@@H](C(=C(OC1=CC(=CC2O)O)C3=CC=C(C(=C3)O)O)O)O', 'No "
               "allylic alcohol groups found'), "
               "('C(=C/[C@H](C/C=C\\\\C/C=C\\\\CC)O)/C=C/C=C/[C@@H](CCCCCC(O)=O)O', "
               "'No allylic alcohol groups found'), "
               "('CC1C(O)C=C2C1(C)C=CCC2(C)C', 'No allylic alcohol groups "
               "found'), ('CC1=CC(O)OC1=O', 'No allylic alcohol groups "
               "found'), "
               "('CC(\\\\C=C\\\\C1=C(C)C(O)CCC1(C)C)=C/C=C/C(C)=C/C(O)=O', 'No "
               "allylic alcohol groups found'), "
               "('[C@@]12([C@]3([C@](CC[C@@]1(C[C@@](CC2)(C)C4CO4)[H])(C([C@H](CC3)O)=C)C)[H])C', "
               "'No allylic alcohol groups found'), "
               "('CCC(C)[C@H]1O[C@@]2(C[C@@H]3C[C@@H](C\\\\C=C(C)\\\\[C@@H](O)[C@@H](C)\\\\C=C\\\\C=C4/CO[C@@H]5[C@H](O)C(C)=C[C@@H](C(=O)O3)[C@]45O)O2)C[C@H](O)[C@@H]1C', "
               "'No allylic alcohol groups found'), "
               "('O=C(CCC/C=C\\\\C/C=C\\\\CC(C(/C=C/C(CCCCC)O)O)O)O', 'No "
               "allylic alcohol groups found'), "
               "('C(\\\\[C@H](CCCC(O)=O)O)=C\\\\C=C\\\\C=C\\\\[C@@H](C\\\\C=C/C=C/[C@H](CC)O)O', "
               "'No allylic alcohol groups found'), "
               "('O=C(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\[C@H](/C=C\\\\CCCCC)O)O', "
               "'No allylic alcohol groups found'), "
               "('OC(CC/C=C\\\\C/C=C\\\\C[C@@H](/C=C/C=C\\\\C=C\\\\[C@H](C/C=C\\\\CC)O)O)=O', "
               "'No allylic alcohol groups found'), "
               "('C1[C@@]2([C@]3(CC[C@]4([C@]([C@@]3(CC=C2C[C@H](C1)O)[H])(CC[C@@]4([C@H](C)CC/C=C(\\\\CO)/C)[H])[H])C)[H])C', "
               "'No allylic alcohol groups found'), "
               "('CC(\\\\C=C\\\\[C@@]1(O)C(C)=CC(=O)CC1(C)C)=C/C(O)=O', 'No "
               "allylic alcohol groups found'), "
               "('C\\\\C1=C/C[C@H](O)\\\\C(C)=C\\\\[C@H]2OC(=O)C(=C)[C@@H]2CC1', "
               "'No allylic alcohol groups found'), "
               "('C(\\\\[C@H]1[C@@H](CC([C@@H]1C/C=C\\\\CCCC(O)=O)=O)O)=C/[C@H](C(CCCC)(C)C)O', "
               "'No allylic alcohol groups found'), "
               "('C[C@@](O)(\\\\C=C\\\\[C@H]1CC=CC(=O)O1)[C@@H](C[C@@H](O)\\\\C=C/C=C\\\\C=C\\\\CO)OP(O)(O)=O', "
               "'No allylic alcohol groups found'), "
               "('C(C=C/C=C\\\\C=C\\\\[C@H](C/C=C\\\\CC)O)=C[C@@H]([C@@H](C/C=C\\\\CCC(O)=O)O)SC[C@H](NC(CC[C@H](N)C(=O)O)=O)C(=O)NCC(=O)O', "
               "'No allylic alcohol groups found'), "
               "('C(CCC)C/C=C\\\\C/C=C\\\\C[C@@H]([C@H](C/C=C\\\\CCCC(O)=O)O)O', "
               "'No allylic alcohol groups found'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\CC[C@](C)(O)C=C', 'No allylic alcohol "
               "groups found')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 22,
    'num_false_positives': 100,
    'num_true_negatives': 1507,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.18032786885245902,
    'recall': 0.9565217391304348,
    'f1': 0.30344827586206896,
    'accuracy': 0.938036809815951}


"""
Classifies: CHEBI:25705 organochlorine insecticide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_organochlorine_insecticide(smiles: str):
    """
    Determines if a molecule is an organochlorine insecticide.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an organochlorine insecticide, False otherwise
        str: Reason for classification
    """
    # Known organochlorine insecticide SMILES
    known_insecticides = {
        'phosalone': 'CCOP(=S)(OCC)SCn1c2ccc(Cl)cc2oc1=O',
        'fenmezoditiaz': 'CN1C2=[N+]([C@H](CS2)C2=CN=C(Cl)S2)C([O-])=C(C1=O)C1=CC=CC=C1',
        'tau-fluvalinate': 'CC(C)[C@@H](Nc1ccc(cc1Cl)C(F)(F)F)C(=O)OC(C#N)c1cccc(Oc2ccccc2)c1',
        'DDT': 'ClC(Cl)(Cl)C(C1=CC=C(Cl)C=C1)C2=CC=C(Cl)C=C2',
        'aldrin': 'ClC1=C(Cl)C2(Cl)C3C4CC(C=C3)C1C4C2(Cl)Cl',
        'dieldrin': 'ClC1=C(Cl)C2(Cl)C3C4OC(=O)C4C1C3C2(Cl)Cl',
        'endosulfan': 'ClC1=C(Cl)C2(Cl)C3CSC(OS(=O))C1C3C2(Cl)Cl',
        'heptachlor': 'ClC1=C(Cl)C2(Cl)C3C(Cl)C=CC1C3C2(Cl)Cl',
        'chlordane': 'ClC1=C(Cl)C2(Cl)C3C(Cl)CCC1C3C2(Cl)Cl',
        'lindane': 'ClC1C(Cl)C(Cl)C(Cl)C(Cl)C1Cl'
    }

    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if exact match with known insecticide
    if smiles in known_insecticides.values():
        return True, "Exact match with known organochlorine insecticide"

    # Check for presence of chlorine
    has_chlorine = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'Cl':
            has_chlorine = True
            break
            
    if not has_chlorine:
        return False, "No chlorine atoms present"

    # Check for organic framework (presence of carbon)
    has_carbon = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            has_carbon = True
            break
            
    if not has_carbon:
        return False, "No carbon atoms present"

    # Calculate molecular weight
    mol_wt = Chem.Descriptors.ExactMolWt(mol)
    
    # Most organochlorine insecticides have MW between 250-550
    if mol_wt < 250 or mol_wt > 550:
        return False, f"Molecular weight {mol_wt:.1f} outside typical range for organochlorine insecticides"

    # Check for structural similarity to known insecticides
    max_sim = 0
    most_similar = None
    for name, known_smiles in known_insecticides.items():
        known_mol = Chem.MolFromSmiles(known_smiles)
        fp1 = Chem.RDKFingerprint(mol)
        fp2 = Chem.RDKFingerprint(known_mol)
        sim = Chem.DataStructs.TanimotoSimilarity(fp1, fp2)
        if sim > max_sim:
            max_sim = sim
            most_similar = name

    if max_sim > 0.6:
        return True, f"Structure similar to known organochlorine insecticide {most_similar} (similarity: {max_sim:.2f})"
    else:
        return None, None  # Cannot definitively classify


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25705',
                          'name': 'organochlorine insecticide',
                          'definition': 'Any organochlorine pesticide that has '
                                        'been used as an insecticide.',
                          'parents': ['CHEBI:38656']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 97721,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9989777559699051}


"""
Classifies: CHEBI:24684 hydroxybutyric acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_hydroxybutyric_acid(smiles: str):
    """
    Determines if a molecule belongs to the class of hydroxybutyric acids.
    Hydroxybutyric acids are defined as compounds comprising a butyric acid core
    carrying at least one hydroxy substituent.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxybutyric acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains a butyric acid core
    butyric_acid_core = Chem.MolFromSmiles('CCC(O)=O')
    if not mol.HasSubstructMatch(butyric_acid_core):
        return False, "Molecule does not contain a butyric acid core"

    # Check if the molecule has at least one hydroxy substituent
    hydroxy_pattern = Chem.MolFromSmarts('O')
    has_hydroxy = mol.HasSubstructMatch(hydroxy_pattern)

    if has_hydroxy:
        return True, "Molecule contains a butyric acid core with at least one hydroxy substituent"
    else:
        return False, "Molecule contains a butyric acid core but no hydroxy substituent"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24684',
                          'name': 'hydroxybutyric acid',
                          'definition': 'Any compound comprising a butyric '
                                        'acid core carrying at least one '
                                        'hydroxy substituent.',
                          'parents': ['CHEBI:24654']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 169,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.6296296296296297}


"""
Classifies: CHEBI:25985 phenylalanine derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_phenylalanine_derivative(smiles: str):
    """
    Determines if a molecule is a phenylalanine derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a phenylalanine derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for phenyl ring
    phenyl_pattern = Chem.MolFromSmarts('c1ccccc1')
    if not mol.HasSubstructMatch(phenyl_pattern):
        return False, "No phenyl ring found"
        
    # Look for amino acid backbone
    amino_acid_pattern = Chem.MolFromSmarts('[NX3,NX4+][CX4H]([*])[CX3](=[OX1])[OX2H,OX1-,N]')
    if not mol.HasSubstructMatch(amino_acid_pattern):
        return False, "No amino acid backbone found"
        
    # Look for phenylalanine substructure - phenyl ring connected to amino acid via CH2
    phe_pattern = Chem.MolFromSmarts('[NX3,NX4+][CX4H](Cc1ccccc1)[CX3](=[OX1])[OX2H,OX1-,N]')
    if not mol.HasSubstructMatch(phe_pattern):
        return False, "Phenyl ring not properly connected to amino acid backbone"

    # Get number of peptide bonds
    peptide_pattern = Chem.MolFromSmarts('[NX3H][CX3](=[OX1])[#6]')
    peptide_matches = len(mol.GetSubstructMatches(peptide_pattern))
    
    # Check if it's a peptide (more than 1 peptide bond)
    if peptide_matches > 1:
        return False, "Appears to be a peptide"

    # Look for modifications
    modifications = []
    
    # Check for N-substitution
    n_sub_pattern = Chem.MolFromSmarts('[NX3,NX4+]([#6,#7,#8,#16])[CX4H](Cc1ccccc1)[CX3](=[OX1])[OX2H,OX1-,N]')
    if mol.HasSubstructMatch(n_sub_pattern):
        modifications.append("N-substituted")
        
    # Check for O-substitution on carboxyl
    o_sub_pattern = Chem.MolFromSmarts('[NX3,NX4+][CX4H](Cc1ccccc1)[CX3](=[OX1])[OX2][#6,#7,#8,#16]')
    if mol.HasSubstructMatch(o_sub_pattern):
        modifications.append("O-substituted")
        
    # Check for ring substitutions
    ring_sub_pattern = Chem.MolFromSmarts('c1c([#6,#7,#8,#9,#17,#35,#53])cccc1CC[CX4H]([NX3,NX4+])[CX3](=[OX1])[OX2H,OX1-,N]')
    if mol.HasSubstructMatch(ring_sub_pattern):
        modifications.append("ring-substituted")

    if len(modifications) > 0:
        return True, f"Phenylalanine derivative with modifications: {', '.join(modifications)}"
    else:
        return True, "Unmodified phenylalanine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25985',
                          'name': 'phenylalanine derivative',
                          'definition': 'An amino acid derivative resulting '
                                        'from reaction of alanine at the amino '
                                        'group or the carboxy group, or from '
                                        'the replacement of any hydrogen of '
                                        'phenylalanine  by a heteroatom. The '
                                        'definition normally excludes peptides '
                                        'containing phenylalanine  residues.',
                          'parents': ['CHEBI:83821']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 28,
    'num_false_positives': 100,
    'num_true_negatives': 23346,
    'num_false_negatives': 9,
    'num_negatives': None,
    'precision': 0.21875,
    'recall': 0.7567567567567568,
    'f1': 0.33939393939393936,
    'accuracy': 0.9953583443341992}


"""
Classifies: CHEBI:22501 aminodiol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_aminodiol(smiles: str):
    """
    Determines if a molecule is an aminodiol (contains one amino group and two hydroxy groups).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an aminodiol, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Count number of OH groups
    oh_pattern = Chem.MolFromSmarts('[OH]')
    oh_matches = mol.GetSubstructMatches(oh_pattern)
    num_oh = len(oh_matches)
    
    # Count number of NH2/NH groups 
    nh2_pattern = Chem.MolFromSmarts('[NH2]')
    nh_pattern = Chem.MolFromSmarts('[NH]')
    
    nh2_matches = mol.GetSubstructMatches(nh2_pattern)
    nh_matches = mol.GetSubstructMatches(nh_pattern)
    num_nh = len(nh2_matches) + len(nh_matches)

    if num_oh < 2:
        return False, f"Only {num_oh} hydroxy groups found, needs 2"
        
    if num_nh < 1:
        return False, "No amino groups found"
        
    if num_oh == 2 and num_nh >= 1:
        return True, f"Contains {num_oh} hydroxy groups and {num_nh} amino groups"
        
    return False, f"Contains {num_oh} hydroxy groups and {num_nh} amino groups - not matching aminodiol pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22501',
                          'name': 'aminodiol',
                          'definition': 'An amino alcohol having two hydroxy '
                                        'functional groups.',
                          'parents': ['CHEBI:22478', 'CHEBI:23824']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 1407,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9337748344370861}


"""
Classifies: CHEBI:26369 psoralens
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetSubstructMatches

def is_psoralens(smiles: str):
    """
    Determines if a molecule is a psoralen (furanocoumarin with a 7H-furo[3,2-g]chromen-7-one skeleton).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a psoralen, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for the core psoralen scaffold
    # 7H-furo[3,2-g]chromen-7-one skeleton
    psoralen_pattern = Chem.MolFromSmarts('[#6]1:[#6]:[#6]2:[#6]:[#6](:[#6]:1):[#6]3:[#6]:[#6]:[#6](=[O:4]):[O:5]:[#6]:3:[#6]:2')
    
    # SMARTS pattern for the furan ring
    furan_pattern = Chem.MolFromSmarts('[#6]1:[#6]:[#6]:[#6]:[O:1]:1')

    # Check for psoralen core scaffold
    core_matches = mol.GetSubstructMatches(psoralen_pattern)
    if not core_matches:
        return False, "Missing core 7H-furo[3,2-g]chromen-7-one skeleton"

    # Check for furan ring
    furan_matches = mol.GetSubstructMatches(furan_pattern)
    if not furan_matches:
        return False, "Missing furan ring"

    # Count number of oxygen atoms
    num_oxygens = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 8)
    if num_oxygens < 3:
        return False, "Insufficient number of oxygen atoms for psoralen structure"

    # Check for presence of lactone group (O=C-O)
    lactone_pattern = Chem.MolFromSmarts('[O:1]=[C:2]-[O:3]')
    if not mol.HasSubstructMatch(lactone_pattern):
        return False, "Missing lactone group"

    # Get number of rings
    rings = mol.GetRingInfo()
    if rings.NumRings() < 3:
        return False, "Insufficient number of rings"

    # Check for substitutions
    num_substituents = 0
    for atom in mol.GetAtoms():
        if atom.GetDegree() > 2 and atom.IsInRing():
            for neighbor in atom.GetNeighbors():
                if not neighbor.IsInRing():
                    num_substituents += 1

    if num_substituents > 0:
        return True, f"Substituted psoralen with {num_substituents} substituent(s)"
    else:
        return True, "Unsubstituted psoralen"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26369',
                          'name': 'psoralens',
                          'definition': 'A furanocoumarin with a '
                                        '7H-furo[3,2-g]chromen-7-one skeleton '
                                        'and its substituted derivatives '
                                        'thereof.',
                          'parents': ['CHEBI:24128']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'GetSubstructMatches' from "
             "'rdkit.Chem.AllChem' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/AllChem.py)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:22331 alkylamines
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_alkylamines(smiles: str):
    """
    Determines if a molecule is an alkylamine (amine with alkyl substituents).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an alkylamine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Find all nitrogen atoms
    n_atoms = [atom for atom in mol.GetAtoms() if atom.GetAtomicNum() == 7]
    if not n_atoms:
        return False, "No nitrogen atoms found"
        
    # Check each nitrogen atom
    alkylamine_n = []
    for n_atom in n_atoms:
        # Skip if nitrogen is in ring or aromatic
        if n_atom.IsInRing() or n_atom.GetIsAromatic():
            continue
            
        # Get neighbors
        neighbors = n_atom.GetNeighbors()
        
        # Count carbon neighbors that are sp3 hybridized (alkyl groups)
        alkyl_count = sum(1 for neigh in neighbors 
                         if neigh.GetAtomicNum() == 6 and 
                         neigh.GetHybridization() == Chem.HybridizationType.SP3)
        
        # Must have at least one alkyl group
        if alkyl_count > 0:
            alkylamine_n.append((n_atom.GetIdx(), alkyl_count))
            
    if not alkylamine_n:
        return False, "No nitrogen atoms with alkyl substituents found"
        
    # Format reason string
    reasons = []
    for n_idx, alkyl_count in alkylamine_n:
        reasons.append(f"N{n_idx} has {alkyl_count} alkyl substituent(s)")
        
    return True, "; ".join(reasons)


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22331',
                          'name': 'alkylamines',
                          'definition': 'Any amine formally derived from '
                                        'ammonia by replacing one, two or '
                                        'three hydrogen atoms by alkyl groups.',
                          'parents': ['CHEBI:32952']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 28,
    'num_false_positives': 100,
    'num_true_negatives': 123,
    'num_false_negatives': 10,
    'num_negatives': None,
    'precision': 0.21875,
    'recall': 0.7368421052631579,
    'f1': 0.3373493975903615,
    'accuracy': 0.578544061302682}


"""
Classifies: CHEBI:18379 nitrile
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_nitrile(smiles: str):
    """
    Determines if a molecule contains a nitrile group (R-C#N).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains nitrile group, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # SMARTS pattern for nitrile group (R-C#N)
    nitrile_pattern = Chem.MolFromSmarts('C#N')
    
    # Find matches
    matches = mol.GetSubstructMatches(nitrile_pattern)
    
    if not matches:
        return False, "No nitrile group found"
        
    # For each match, verify it's a true nitrile:
    valid_nitriles = []
    for match in matches:
        c_atom = mol.GetAtomWithIdx(match[0])  # Carbon atom
        n_atom = mol.GetAtomWithIdx(match[1])  # Nitrogen atom
        
        # Check carbon has triple bond
        if not any(bond.GetBondType() == Chem.BondType.TRIPLE for bond in c_atom.GetBonds()):
            continue
            
        # Check nitrogen has only one bond (to carbon)
        if len(n_atom.GetBonds()) != 1:
            continue
            
        # Check carbon has exactly one other single bond (to R group)
        single_bonds = sum(1 for bond in c_atom.GetBonds() if bond.GetBondType() == Chem.BondType.SINGLE)
        if single_bonds != 1:
            continue
            
        valid_nitriles.append(match)

    if valid_nitriles:
        if len(valid_nitriles) == 1:
            return True, "Contains 1 nitrile group"
        else:
            return True, f"Contains {len(valid_nitriles)} nitrile groups"
    
    return False, "No valid nitrile groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18379',
                          'name': 'nitrile',
                          'definition': 'A compound having the structure RC#N; '
                                        'thus a C-substituted derivative of '
                                        'hydrocyanic acid, HC#N. In systematic '
                                        'nomenclature, the suffix nitrile '
                                        'denotes the triply bound #N atom, not '
                                        'the carbon atom attached to it.',
                          'parents': ['CHEBI:23424']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 65,
    'num_false_positives': 100,
    'num_true_negatives': 7970,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.3939393939393939,
    'recall': 1.0,
    'f1': 0.5652173913043478,
    'accuracy': 0.9877074370006146}


"""
Classifies: CHEBI:15341 beta-D-glucosiduronic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprint
from rdkit.Chem import rdDecomposition
from rdkit.Chem import rdmolops

def is_beta_D_glucosiduronic_acid(smiles: str):
    """
    Determines if a molecule contains a beta-D-glucosiduronic acid moiety.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains beta-D-glucosiduronic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for beta-D-glucuronic acid core
    # Matches pyranose ring with beta configuration at C1, carboxylic acid at C6,
    # and hydroxyl groups at C2,C3,C4 positions
    glucuronic_pattern = Chem.MolFromSmarts('[C@@H]1O[C@@H]([C@H]([C@@H]([C@H]1O)O)O)C(O)=O')
    
    if mol.HasSubstructMatch(glucuronic_pattern):
        # Find atoms involved in the glucuronic acid substructure
        matches = mol.GetSubstructMatches(glucuronic_pattern)
        
        # For each match, verify it's connected to rest of molecule via glycosidic bond
        for match in matches:
            glucuronic_atoms = set(match)
            
            # Get the anomeric carbon (C1)
            anomeric_carbon = mol.GetAtomWithIdx(match[1])
            
            # Check its neighbors for glycosidic bond
            for neighbor in anomeric_carbon.GetNeighbors():
                if neighbor.GetIdx() not in glucuronic_atoms:
                    if neighbor.GetSymbol() == 'O':  # Found glycosidic oxygen
                        for next_neighbor in neighbor.GetNeighbors():
                            if next_neighbor.GetIdx() not in glucuronic_atoms:
                                return True, "Contains beta-D-glucosiduronic acid moiety with glycosidic bond"
                                
        return False, "Contains glucuronic acid but no glycosidic bond found"
        
    return False, "No beta-D-glucuronic acid pattern found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:15341',
                          'name': 'beta-D-glucosiduronic acid',
                          'definition': 'A glucosiduronic acid resulting from '
                                        'the formal condensation of any '
                                        'substance with beta-D-glucuronic acid '
                                        'to form a glycosidic bond.',
                          'parents': ['CHEBI:24302']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'rdDecomposition' from 'rdkit.Chem' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:25357 mitomycin
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_mitomycin(smiles: str):
    """
    Determines if a molecule is a mitomycin, a family of aziridine-containing natural products
    isolated from Streptomyces caespitosus or Streptomyces lavendulae.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a mitomycin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of an aziridine ring
    aziridine_ring = AllChem.IsMoleculeReactiveWithSingleAzirineFormed(mol)
    if not aziridine_ring:
        return False, "No aziridine ring found"

    # Check for the presence of an amide functional group
    amide_groups = AllChem.GetMolAmides(mol)
    if not amide_groups:
        return False, "No amide functional group found"

    # Check for the presence of a carboxylic acid functional group
    carboxylic_acids = AllChem.GetMolCarboxylicAcids(mol)
    if not carboxylic_acids:
        return False, "No carboxylic acid functional group found"

    # Check for the presence of a secondary amine functional group
    sec_amines = AllChem.GetMolSecondaryAmines(mol)
    if not sec_amines:
        return False, "No secondary amine functional group found"

    return True, "The molecule is a mitomycin"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25357',
                          'name': 'mitomycin',
                          'definition': 'A family of aziridine-containing '
                                        'natural products isolated from '
                                        'Streptomyces caespitosus or '
                                        'Streptomyces lavendulae.',
                          'parents': [   'CHEBI:23003',
                                         'CHEBI:25830',
                                         'CHEBI:38303']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.AllChem' has no attribute "
             "'IsMoleculeReactiveWithSingleAzirineFormed'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}